package ij.measure;

/** This class consists of public fields that describe the
	spatial calibration of an image. */
   
public class Calibration {

	public static final int NONE=20,INVERTER=21,UNCALIBRATED_OD=22;

	/** Pixel width in 'units'. */
	public double pixelWidth = 1.0;
	
	/** Pixel height in 'units'. */
	public double pixelHeight = 1.0;
	
	/** Pixel (voxel) depth in 'units'. */
	public double pixelDepth = 1.0;

	/** Measurement unit (e.g. "inch", "cm"). */
	public String unit = "pixel";
	
	/** Measurement units. (e.g. "inches", "cm")*/
	public String units = "pixels";
	
	/** Calibration function coefficients. */
	public double[] coefficients;
		
	/** Calibration function ID. */
	private int function = NONE;

	/** Calibration table. */
	private float[] cTable;


	/** Constructs a new Scale object using the default values. */ 
	public Calibration() {
	}
	
	/** Returns true if this image is spatially calibrated. */
	public boolean scaled() {
		return pixelWidth!=1.0 || pixelHeight!=1.0;
	}
	
 	/** Returns true if this image is density calibrated. */
	public boolean calibrated() {
		return coefficients!=null;
	}
	
	/** Returns the calibration function ID. */
 	public int getFunction() {
 		return function;
 	}
 	
	/** Returns the calibration table. */
 	public float[] getCTable() {
 		return cTable;
 	}
 	
	/** Sets the calibration table. */
 	public void setCTable(float[] table, int function) {
 		cTable = table;
 		this.function = function;
 	}

  	/** Converts a raw pixel value to a density calibrated value. */
 	public double getCValue(int value) {
		//ij.IJ.write("getCValue: "+this);
 		if (cTable!=null && value>=0 && value<cTable.length)
 			return cTable[value];
 		else
 			return value;
 	}
 	
	/** Returns a clone of this object. */
	public Calibration copy() {
		Calibration copy = new Calibration();
		copy.pixelWidth = pixelWidth;
		copy.pixelHeight = pixelHeight;
		copy.pixelDepth = pixelDepth;
		copy.unit = unit;
		copy.units = units;
		copy.function = function;
		copy.cTable = cTable;
		return copy;
	}
	
   public String toString() {
    	return
    		"w=" + pixelWidth
			+ ", h=" + pixelHeight
			+ ", d=" + pixelDepth
			+ ", unit=" + unit
			+ ", units=" + units
 			+ ", cTable=" + (cTable!=null?""+cTable.length:"null");
   }
}


